/*
  angular-dss.js
  Angular service that provides connection to the DSS webservices (JSON, Error Logs)
*/

var dss = angular.module('dSS', []);
dss.service('dss',['$http','$q', function($http,$q) {
    function async(url) {
	return $http.get('/json'+url)
	    .then(function(response) {
		if(!response.data.ok) {
		    throw "DSS JSON request error: "+response.data.message+" (url: /json"+url+")";
		}
		else {
		    return response.data.result;
		}
	    });
    }

    function asyncChild(url,child) {
	return $http.get('/json'+url)
	    .then(function(response) {
		if(!response.data.ok) {
		    throw "DSS JSON request error: "+response.data.message+" (url: /json"+url+")";
		}
		else {
		    return response.data.result[child];
		}
	    });
    }

    this.getLogfile = function(logfile) {
	return $http.get('/logs'+logfile)
	    .then(function(response) {
		if(response.status!=200) {
		    throw new Error("Getting logfiles failed: "+statusText);
		}
		else {
		    var allTextLines = response.data.split(/\r\n|\n/);
		    var lines = [];

		    for (var i=0; i<allTextLines.length; i++) {
			var line = allTextLines[i].split(';');
			if(allTextLines[i].length==0) {
			    continue;
			}
			/* Parse first column (date/time) with format from Date.toISOString():
			   [2015-06-15T14:14:54.633+01:00]
			*/
			line[0] = new Date(line[0].substr(1,29));
			lines.push(line);
		    }
		    return lines.reverse();
		}
	    });
    };

    this.Property = {
	getChildrenValues: function(path) {
	    return this.getChildren(path).then(function(children) {
		var childrenPromises = children.map(function(child) {
		    if(child.type=="none") {
			return true;
		    }
		    var method;
		    if(child.type=="string") {
			method = "getString";
		    }
		    else if(child.type=="integer") {
			method = "getInteger";
		    }
		    else if(child.type=="boolean") {
			method = "getBoolean";
		    }
		    else {
			throw new Error("Invalid data type of variable "+child.name);
		    }
		    return asyncChild('/property/'+method+'?path='+path+'/'+child.name,"value");
		});
		return $q.all(childrenPromises).then(function(values) {
		    var result = {};
		    for(var i=0;i<children.length;i++) {
    			result[children[i].name] = values[i];
		    }
		    return result;
		});
	    });
	},
	getChildren: function(path) {
	    return async('/property/getChildren?path='+path);
	},
	getString: function(path) {
	    return asyncChild('/property/getString?path='+path,"value");
	},
	setString: function(path,value) {
	    return async('/property/setString?path='+path+'&value='+value);
	},
	getInteger: function(path) {
	    return asyncChild('/property/getInteger?path='+path,"value");
	},
	setInteger: function(path,value) {
	    return async('/property/setInteger?path='+path+'&value='+value);
	},
	getBoolean: function(path) {
	    return asyncChild('/property/getBoolean?path='+path,"value");
	},
	setBoolean: function(path,value) {
	    return async('/property/setBoolean?path='+path+'&value='+(value?'true':'false'));
	},
    };
    this.Apartment = {
	getStructure: function(path) {
	    return async('/apartment/getStructure');
	},
	callScene: function(sceneNumber,groupID) {
	    return async('/apartment/callScene?sceneNumber='+sceneNumber+(groupID?'&groupID='+groupID:''));
	},
	getDevices: function(path) {
	    return async('/apartment/getDevices');
	},
    };
    this.Device = {
	getName: function(dsid) {
	    return asyncChild('/device/getName?dsid='+dsid,"name");
	},
    };
    this.Event = {
	raise: function(name, parameter) {
	    var paramstr="";
	    for (var key in parameter) {
  		if (parameter.hasOwnProperty(key)) {
		    paramstr += key+"="+parameter[key]+";";
		}
	    };
	    return async('/event/raise?name='+name+"&parameter="+paramstr.substring(0,paramstr.length-1));
	},
    };
    this.Scene = { //@Diego
	disableScene: function (dsid,sceneID){
	    return async('/device/setSceneMode?dsid='+dsid+'&sceneID='+sceneID+'&dontCare=1');
	}
    };

}]);
